@@ -3,17 +3,19 @@
 
 // gmp.h calls cstddef with __need_size_t defined
 #include <cstddef>
-// and avoid the inclusion of stdlib.h
 #include <cstdlib>
 #include <cmath>
+
 #include <gmp.h>
 
 
-
 #include <R.h>
+
 #include <Rinternals.h>
+#ifndef AS_INTEGER
 // the only thing we use from <Rdefines.h> :
-#define AS_INTEGER(x) coerceVector(x,INTSXP)
+#define AS_INTEGER(x) Rf_coerceVector(x,INTSXP)
+#endif
 
 #define class_P(_x_) CHAR(Rf_asChar(Rf_getAttrib(_x_, R_ClassSymbol)))
 
